package com.ycwl.basic.device;

import com.ycwl.basic.device.checker.IDeviceStatusChecker;
import com.ycwl.basic.device.checker.impl.AliOssDeviceChecker;
import com.ycwl.basic.device.checker.impl.AlwaysOnDeviceChecker;
import com.ycwl.basic.device.enums.DeviceStoreTypeEnum;
import com.ycwl.basic.device.operator.IDeviceStorageOperator;
import com.ycwl.basic.device.operator.impl.AliOssStorageOperator;
import com.ycwl.basic.device.operator.impl.LocalStorageOperator;
import com.ycwl.basic.model.pc.device.entity.DeviceConfigEntity;
import com.ycwl.basic.model.pc.device.entity.DeviceEntity;

public class DeviceFactory {
    public static IDeviceStorageOperator getDeviceStorageOperator(DeviceEntity device, DeviceConfigEntity config) {
        IDeviceStorageOperator operator = null;
        if (config.getStoreType() == DeviceStoreTypeEnum.ALI_OSS.getType()) {
            operator = new AliOssStorageOperator(config.getStoreConfigJson());
        } else if (config.getStoreType() == DeviceStoreTypeEnum.LOCAL.getType()) {
            operator = new LocalStorageOperator(config.getStoreConfigJson());
        }
        if (operator == null) {
            return null;
        }
        operator.setDevice(device);
        operator.setDeviceConfig(config);
        return operator;
    }

    public static IDeviceStatusChecker getDeviceStatusChecker(DeviceEntity device, DeviceConfigEntity config) {
        IDeviceStatusChecker checker = null;
        if (config.getOnlineCheck() <= 0) {
            checker = new AlwaysOnDeviceChecker();
        } else {
            if (config.getStoreType() == DeviceStoreTypeEnum.ALI_OSS.getType()) {
                checker = new AliOssDeviceChecker(config.getStoreConfigJson());
            }
        }
        if (checker == null) {
            return null;
        }
        checker.setDevice(device);
        checker.setDeviceConfig(config);
        return checker;
    }
}
