package com.ycwl.basic.service.impl.task;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.facebody.model.v20191230.AddFaceEntityRequest;
import com.aliyuncs.facebody.model.v20191230.AddFaceRequest;
import com.aliyuncs.facebody.model.v20191230.AddFaceResponse;
import com.aliyuncs.facebody.model.v20191230.CreateFaceDbRequest;
import com.aliyuncs.facebody.model.v20191230.DeleteFaceDbRequest;
import com.aliyuncs.facebody.model.v20191230.DeleteFaceEntityRequest;
import com.aliyuncs.facebody.model.v20191230.ListFaceEntitiesRequest;
import com.aliyuncs.facebody.model.v20191230.ListFaceEntitiesResponse;
import com.aliyuncs.facebody.model.v20191230.SearchFaceRequest;
import com.aliyuncs.facebody.model.v20191230.SearchFaceResponse;
import com.ycwl.basic.config.FaceDetectConfig;
import com.ycwl.basic.exception.BaseException;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.mapper.FaceSampleMapper;
import com.ycwl.basic.mapper.ScenicMapper;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.pc.faceSample.entity.FaceSampleEntity;
import com.ycwl.basic.model.pc.faceSample.req.FaceSampleReqQuery;
import com.ycwl.basic.model.pc.faceSample.resp.FaceSampleRespVO;
import com.ycwl.basic.model.pc.scenic.entity.ScenicConfigEntity;
import com.ycwl.basic.model.task.resp.AddFaceRespVo;
import com.ycwl.basic.model.task.resp.SearchFaceRespVo;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.utils.AliFaceUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.profile.DefaultProfile;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TaskFaceServiceImpl implements TaskFaceService {
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private FaceSampleMapper faceSampleMapper;
    @Autowired
    private ScenicMapper scenicMapper;
    @Autowired
    private FaceDetectConfig faceDetectConfig;

    private IAcsClient getClient() {
        DefaultProfile profile = DefaultProfile.getProfile(
                faceDetectConfig.getRegion(),faceDetectConfig.getAccessKeyId(), faceDetectConfig.getAccessKeySecret());
        IAcsClient client = new DefaultAcsClient(profile);
        return client;
    }
    @Override
    public SearchFaceRespVo searchFace(Long scenicId, Long faceId) {
        FaceRespVO faceRespVO = faceMapper.getById(faceId);
        SearchFaceRespVo respVo = searchFace(scenicId, faceRespVO.getFaceUrl());
        if (respVo != null) {
            FaceEntity faceEntity = new FaceEntity();
            faceEntity.setId(faceId);
            faceEntity.setMatchResult(respVo.getSearchResultJson());
            faceEntity.setFirstMatchRate(BigDecimal.valueOf(respVo.getFirstMatchRate()));
            faceEntity.setScore(respVo.getScore());
            faceEntity.setMatchSampleIds(StringUtils.join(respVo.getSampleListIds(), ","));
            faceMapper.update(faceEntity);
        }
        return respVo;
    }

    @Override
    public SearchFaceRespVo searchFace(Long scenicId, String faceUrl) {
        IAcsClient client = getClient();
        SearchFaceRequest request = new SearchFaceRequest();
        request.setDbName(scenicId.toString());
        request.setImageUrl(faceUrl);
        request.setLimit(100);
        request.setQualityScoreThreshold(80F);
        try {
            SearchFaceResponse response = client.getAcsResponse(request);
            List<SearchFaceResponse.Data.MatchListItem> matchList = response.getData().getMatchList();
            if (matchList.isEmpty()) {
                return null;
            }
            SearchFaceRespVo respVo = new SearchFaceRespVo();
            respVo.setScore(matchList.get(0).getQualitieScore());
            respVo.setSearchResultJson(JSON.toJSONString(matchList.get(0)));
            if (matchList.get(0).getFaceItems().isEmpty()) {
                return respVo;
            }
            List<SearchFaceResponse.Data.MatchListItem.FaceItemsItem> faceItems = matchList.get(0).getFaceItems().stream()
                    .filter(faceItemsItem -> faceItemsItem.getConfidence() > 50).collect(Collectors.toList());
            List<Long> faceSampleIds = faceItems.stream()
                    .map(SearchFaceResponse.Data.MatchListItem.FaceItemsItem::getExtraData)
                    .map(Long::parseLong)
                    .collect(Collectors.toList());
            respVo.setFirstMatchRate(matchList.get(0).getFaceItems().get(0).getConfidence());
            respVo.setSampleListIds(faceSampleIds);
            return respVo;
        } catch (Exception e) {
            log.error("人脸搜索失败：{}", e.getMessage());
            throw new BaseException(e.getMessage());
        }
    }

    @Override
    public AddFaceRespVo addFaceSample(Long faceSampleId) {
        FaceSampleRespVO faceSampleRespVO = faceSampleMapper.getById(faceSampleId);
        AddFaceEntityRequest request = new AddFaceEntityRequest();
        request.setDbName(faceSampleRespVO.getScenicId().toString());
        String entityId = AliFaceUtil.generateEntityId(faceSampleRespVO);
        request.setEntityId(entityId);
        IAcsClient client = getClient();
        try {
            client.getAcsResponse(request);
        } catch (ClientException e) {
            return null;
        }
        AddFaceRequest addFaceRequest = new AddFaceRequest();
        addFaceRequest.setDbName(faceSampleRespVO.getScenicId().toString());
        addFaceRequest.setEntityId(entityId);
        addFaceRequest.setImageUrl(faceSampleRespVO.getFaceUrl());
        addFaceRequest.setExtraData(faceSampleId.toString());
        AddFaceRespVo respVo = new AddFaceRespVo();
        try {
            AddFaceResponse acsResponse = client.getAcsResponse(addFaceRequest);
            FaceSampleEntity faceSampleEntity = new FaceSampleEntity();
            faceSampleEntity.setId(faceSampleId);
            faceSampleEntity.setScore(acsResponse.getData().getQualitieScore());
            faceSampleEntity.setUpdateAt(new Date());
            faceSampleMapper.update(faceSampleEntity);
            respVo.setScore(acsResponse.getData().getQualitieScore());
        } catch (ClientException e) {
            return null;
        }
        return respVo;
    }

    @Override
    public void batchDeleteFace(Long scenicId) {
        FaceSampleReqQuery query = new FaceSampleReqQuery();
        query.setDeviceId(scenicId);
        faceSampleMapper.list(query);
        ScenicConfigEntity scenicConfig = scenicMapper.getConfig(scenicId);
        if (scenicConfig == null) {
            return;
        }
        Integer sampleStoreDay = scenicConfig.getSampleStoreDay();
        if (sampleStoreDay == null) {
            sampleStoreDay = 3;
        }
        Date thatDay = DateUtil.offsetDay(new Date(), -sampleStoreDay);
        Date dayStart = DateUtil.beginOfDay(thatDay);
        Date dayEnd = DateUtil.endOfDay(thatDay);
        query.setStartTime(dayStart);
        query.setEndTime(dayEnd);
        IAcsClient client = getClient();
        faceSampleMapper.list(query).forEach(faceSampleEntity -> {
            String entityId = AliFaceUtil.generateEntityId(faceSampleEntity);
            DeleteFaceEntityRequest request = new DeleteFaceEntityRequest();
            request.setDbName(scenicId.toString());
            request.setEntityId(entityId);
            try {
                client.getAcsResponse(request);
            } catch (ClientException e) {
                return;
            }
        });
    }

    @Override
    public void deleteFaceDB(Long scenicId) {
        ListFaceEntitiesRequest request = new ListFaceEntitiesRequest();
        request.setDbName(scenicId.toString());
        request.setLimit(200);
        IAcsClient client = getClient();
        try {
            while (true) {
                ListFaceEntitiesResponse response = client.getAcsResponse(request);
                if (response.getData().getTotalCount() == 0) {
                    break;
                }
                response.getData().getEntities().forEach(entity -> {
                    DeleteFaceEntityRequest deleteFaceEntityRequest = new DeleteFaceEntityRequest();
                    deleteFaceEntityRequest.setDbName(entity.getDbName());
                    deleteFaceEntityRequest.setEntityId(entity.getEntityId());
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    try {
                        client.getAcsResponse(deleteFaceEntityRequest);
                    } catch (ClientException e) {
                        log.error("删除人脸数据失败！", e);
                    }
                });
            }
            DeleteFaceDbRequest deleteFaceDbRequest = new DeleteFaceDbRequest();
            deleteFaceDbRequest.setName(scenicId.toString());
            try {
                client.getAcsResponse(deleteFaceDbRequest);
            } catch (ClientException e) {
                log.error("删除人脸数据库失败！", e);
            }
        } catch (ClientException e) {
            log.error("删除人脸数据库失败！", e);
        }
    }

    @Override
    public void createFaceDB(Long scenicId) {
        try {
            CreateFaceDbRequest request = new CreateFaceDbRequest();
            request.setName(scenicId.toString());
            IAcsClient client = getClient();
            client.getAcsResponse(request);
        } catch (ClientException e) {
            log.error("阿里云添加人脸数据库失败！", e);
        }
    }
}
